port module Counter exposing (Flags, Model, Msg, encodeMsg, fromCache, init, msgDecoder, subscriptions, update, view)

import AMR
import Browser
import Html exposing (button, div, text)
import Html.Events exposing (onClick)
import Json.Decode as Decode
import Json.Encode as Encode


type alias Flags =
    { amr : Maybe String }


fromCache : Flags -> Maybe String
fromCache =
    .amr


type alias Model =
    Int


type Msg
    = Increment
    | Decrement
    | DoNothing


encodeMsg : Msg -> Encode.Value
encodeMsg msg =
    case msg of
        Increment ->
            Encode.string "Increment"

        Decrement ->
            Encode.string "Decrement"

        DoNothing ->
            Encode.string "DoNothing"


msgDecoder : Decode.Decoder Msg
msgDecoder =
    Decode.andThen
        (\text ->
            case text of
                "Increment" ->
                    Decode.succeed Increment

                "Decrement" ->
                    Decode.succeed Decrement

                "DoNothing" ->
                    Decode.succeed DoNothing

                _ ->
                    Decode.fail ("'" ++ text ++ "' is not a valid message!")
        )
        Decode.string


init : a -> ( Int, Cmd Msg )
init =
    always ( 0, Cmd.none )


update : Msg -> Model -> ( Model, Cmd Msg )
update msg count =
    case msg of
        DoNothing ->
            ( count, Cmd.none )

        Increment ->
            ( count + 1, Cmd.none )

        Decrement ->
            ( count - 1, Cmd.none )


subscriptions : a -> Sub Msg
subscriptions =
    always Sub.none


view : Model -> Browser.Document Msg
view count =
    { title = "Counter"
    , body =
        [ button [ onClick Increment ] [ text "+" ]
        , div [] [ text (String.fromInt count) ]
        , button [ onClick Decrement ] [ text "-" ]
        ]
    }
